@@ -1,7 +1,7 @@
 /* This file is part of the KDE project
 
    Copyright (C) 2004 Dario Massarin <nekkar@libero.it>
-   Copyright (C) 2008 Lukas Appelhans <l.appelhans@gmx.de>
+   Copyright (C) 2008 - 2009 Lukas Appelhans <l.appelhans@gmx.de>
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
@@ -16,7 +16,6 @@
 #include "core/transferhandler.h"
 #include "core/plugin/transferfactory.h"
 #include "core/scheduler.h"
-#include "core/transferhistorystore.h"
 
 #include <kiconloader.h>
 #include <klocale.h>
@@ -28,42 +27,86 @@
 #include "nepomukhandler.h"
 #endif
 
+struct StatusStrings
+{
+    const char * context;
+    const char * name;
+};
+
+const StatusStrings STATUSTEXTS[] = {
+    {"", I18N_NOOP("Downloading....")},
+    {I18N_NOOP2_NOSTRIP("transfer state: delayed", "Delayed")},
+    {I18N_NOOP2_NOSTRIP("transfer state: stopped", "Stopped")},
+    {I18N_NOOP2_NOSTRIP("transfer state: aborted", "Aborted")},
+    {I18N_NOOP2_NOSTRIP("transfer state: finished", "Finished")},
+    {"", ""},//TODO: Add FinishedKeepAlive status
+    {I18N_NOOP2_NOSTRIP("changing the destination of the file", "Changing destination")}
+};
+const QStringList STATUSICONS = QStringList() << "media-playback-start" << "view-history" << "process-stop" << "dialog-error" << "dialog-ok" << "media-playback-start" << "media-playback-pause";
+
 Transfer::Transfer(TransferGroup * parent, TransferFactory * factory,
                    Scheduler * scheduler, const KUrl & source, const KUrl & dest,
                    const QDomElement * e)
-    : Job(parent, scheduler),
+    : Job(scheduler, parent),
       m_source(source), m_dest(dest),
       m_totalSize(0), m_downloadedSize(0), m_uploadedSize(0),
       m_percent(0), m_downloadSpeed(0), m_uploadSpeed(0),
       m_uploadLimit(0), m_downloadLimit(0), m_isSelected(false),
       m_visibleUploadLimit(0), m_visibleDownloadLimit(0), m_ratio(0),
       m_handler(0), m_factory(factory)
-{
 #ifdef HAVE_NEPOMUK
-    m_nepomukHandler = new NepomukHandler(this, 0);
+      , m_nepomukHandler(0)
 #endif
-
-    if( e )
-        load( *e );
-    else
-    {
-        setStatus(status(), i18nc("transfer state: stopped", "Stopped"), SmallIcon("process-stop"));
-        setStartStatus(status());
-    }
+{
+    Q_UNUSED(e)
 }
 
 Transfer::~Transfer()
 {
-    TransferHistoryStore::getStore()->saveItem(TransferHistoryItem(*this));
+}
 
-    if(status() == Job::Delayed)
-        m_scheduler->stopDelayTimer(this);
+void Transfer::create()
+{
+#ifdef HAVE_NEPOMUK
+    if (!m_nepomukHandler)
+        m_nepomukHandler = new NepomukHandler(this);
+#endif
 
-    delete(m_handler);
+    init();
+}
+
+void Transfer::destroy()
+{
+    deinit();
+    
+#ifdef HAVE_NEPOMUK
+    nepomukHandler()->deinit();
+#endif //HAVE_NEPOMUK
+}
+
+void Transfer::init()//TODO think about e, maybe not have it at all in the constructor?
+{
+
+}
 
 #ifdef HAVE_NEPOMUK
-    delete(m_nepomukHandler);
-#endif
+void Transfer::setNepomukHandler(NepomukHandler *handler)
+{
+    if (m_nepomukHandler)
+    {
+        delete(m_nepomukHandler);
+        m_nepomukHandler = 0;
+    }
+    m_nepomukHandler = handler;
+}
+#endif //HAVE_NEPOMUK
+
+bool Transfer::setDirectory(const KUrl& newDirectory)
+{
+    Q_UNUSED(newDirectory)
+
+    //the standard implemention always returns false
+    return false;
 }
 
 int Transfer::elapsedTime() const
@@ -74,6 +117,26 @@ int Transfer::elapsedTime() const
     return m_runningSeconds;
 }
 
+int Transfer::averageDownloadSpeed() const
+{
+    const int runningSeconds = elapsedTime();
+    if (runningSeconds)
+    {
+        return m_totalSize / runningSeconds;
+    }
+
+    return 0;
+}
+
+QHash<KUrl, QPair<bool, int> > Transfer::availableMirrors(const KUrl &file) const
+{
+    Q_UNUSED(file)
+
+    QHash<KUrl, QPair<bool, int> > available;
+    available[m_source] = QPair<bool, int>(true, 1);
+    return available;
+}
+
 void Transfer::setUploadLimit(int ulLimit, SpeedLimit limit)
 {
     if (limit == Transfer::VisibleSpeedLimit)
@@ -131,22 +194,6 @@ void Transfer::checkShareRatio()
         setDownloadLimit(0, Transfer::InvisibleSpeedLimit);
 }
 
-void Transfer::setDelay(int seconds)
-{
-    m_scheduler->startDelayTimer(this, seconds);
-
-    setStatus(Job::Delayed, i18nc("transfer state: delayed", "Delayed"), SmallIcon("view-history"));
-
-    setTransferChange(Tc_Status, true);
-}
-
-void Transfer::delayTimerEvent()
-{
-    setStatus(Job::Stopped, i18nc("transfer state: stopped", "Stopped"), SmallIcon("process-stop"));
-
-    setTransferChange(Tc_Status, true);
-}
-
 void Transfer::setLog(const QString& message, LogLevel level)
 {
     QString msg("<font color=\"blue\">" + QTime::currentTime().toString() + "</font> : ");
@@ -190,8 +237,17 @@ void Transfer::save(const QDomElement &element)
     e.setAttribute("Policy", policy() == Job::Start ? "Start" : (policy() == Job::Stop ? "Stop" : "None"));
 }
 
-void Transfer::load(const QDomElement &e)
+void Transfer::load(const QDomElement *element)
 {
+    if (!element)
+    {
+        setStatus(status(), i18nc("transfer state: stopped", "Stopped"), SmallIcon("process-stop"));
+        setStartStatus(status());
+        return;
+    }
+
+    const QDomElement e = *element;
+
     m_source = KUrl(e.attribute("Source"));
     m_dest = KUrl(e.attribute("Dest"));
 
@@ -206,7 +262,7 @@ void Transfer::load(const QDomElement &e)
 
     if((m_totalSize == m_downloadedSize) && (m_totalSize != 0))
     {
-        setStatus(Job::Finished, i18nc("transfer state: finished", "Finished"), SmallIcon("dialog-ok"));
+        setStatus(Job::Finished);
     }
     else
     {
@@ -237,12 +293,20 @@ void Transfer::load(const QDomElement &e)
 
 void Transfer::setStatus(Job::Status jobStatus, const QString &text, const QPixmap &pix)
 {
-    //If a job is finished don't let it to be changed
-    if((status() == Job::Finished) && (jobStatus != Job::Finished))
-        return;
+    QString statusText = text;
+    if (statusText.isEmpty())
+    {
+        statusText = i18nc(STATUSTEXTS[jobStatus].context, STATUSTEXTS[jobStatus].name);
+    }
 
-    m_statusText = text;
-    m_statusPixmap = pix;
+    QPixmap statusIcon = pix;
+    if (statusIcon.isNull())
+    {
+        statusIcon = SmallIcon(STATUSICONS[jobStatus]);
+    }
+
+    m_statusText = statusText;
+    m_statusPixmap = statusIcon;
 
 #ifdef HAVE_NEPOMUK
     if (jobStatus == Job::Finished)
@@ -280,3 +344,13 @@ void Transfer::setTransferChange(ChangesFlags change, bool postEvent)
     }
     handler()->setTransferChange(change, postEvent);
 }
+
+QString Transfer::statusText(Job::Status status)
+{
+    return i18nc(STATUSTEXTS[status].context, STATUSTEXTS[status].name);
+}
+
+QPixmap Transfer::statusPixmap(Job::Status status)
+{
+    return SmallIcon(STATUSICONS[status]);
+}
