@@ -18,7 +18,7 @@
 #include <KDebug>
 
 #include "jobqueue.h"
-#include "kget_export.h"
+#include "../kget_export.h"
 #include "transfer.h"
 
 class QDomElement;
@@ -40,6 +40,7 @@ class TransferTreeModel;
  */
 class KGET_EXPORT TransferGroup : public JobQueue
 {
+    Q_OBJECT
     public:
         enum GroupChange
         {
@@ -57,7 +58,7 @@ class KGET_EXPORT TransferGroup : public JobQueue
 
         typedef int ChangesFlags;
 
-        TransferGroup(TransferTreeModel * model, Scheduler * scheduler, const QString & name=QString());
+        TransferGroup(TransferTreeModel * model, Scheduler * parent, const QString & name=QString());
 
         virtual ~TransferGroup();
 
@@ -282,13 +283,6 @@ class KGET_EXPORT TransferGroup : public JobQueue
          */
         void calculateUploadLimit();
 
-        /**
-         * Notifies that the given transfer has changed
-         *
-         * @param transfer The transfer that has changed
-         */
-        void transferChangedEvent(Transfer * transfer);
-
         /**
          * Saves this group object to the given QDomNode 
          *
