@@ -17,15 +17,20 @@
 #include <kxmlguiwindow.h>
 #include <kurl.h>
 
-#ifdef HAVE_KNOTIFICATIONITEM
-#include <knotificationitem.h>
-#endif
+#include <kstatusnotifieritem.h>
 #include "ui/tray.h"
+#include "core/transfer.h"
+#include "core/transfergroup.h"
+
+#ifdef HAVE_QCA2
+#include <QtCrypto>
+#endif
 
 class ViewsContainer;
 class DropTarget;
-class DBusModelObserver;
+class DBusKgetWrapper;
 class HttpServer;
+class KGet;
 
 /**
  * The main window of KGet.
@@ -34,27 +39,14 @@ class HttpServer;
  */
 class MainWindow : public KXmlGuiWindow
 {
+    friend class DBusKGetWrapper;
+
 Q_OBJECT
 public:
-    explicit MainWindow(bool showMainwindow = true, bool startWithoutAnimation = false, QWidget *parent = 0);
+    explicit MainWindow(bool showMainwindow = true, bool startWithoutAnimation = false, bool doTesting = false, QWidget *parent = 0);
     ~MainWindow();
 
-    // from the DBUS interface
-    virtual void addTransfer(const QString& src, const QString& destDir = QString(), 
-                              bool start = false);
-    virtual void showNewTransferDialog(const QStringList &urls);
-    virtual bool dropTargetVisible() const;
-    virtual void setDropTargetVisible( bool setVisible );
-    virtual void setOfflineMode( bool online );
-    virtual bool offlineMode() const;
-    virtual QVariantMap transfers() const;
-    virtual int transfersSpeed() const;
     virtual void setSystemTrayDownloading(bool running);
-    virtual void importLinks(const QList <QString> &links);
-
-#ifndef HAVE_KNOTIFICATIONITEM
-    KSystemTrayIcon *systemTray() const { return m_dock;};
-#endif
 
 public slots:
     void slotQuit();
@@ -96,6 +88,7 @@ private slots:
     void slotShowMenubar();
     void slotTransferGroupSettings();
     void slotTransferSettings();
+    void slotCreateMetalink();
 
     // transfers slots
     void slotStopAllDownload();
@@ -120,6 +113,10 @@ private slots:
 
     // import links slots
     void slotShowListLinks();
+    
+    //Model changes
+    void slotTransfersChanged(QMap<TransferHandler*, Transfer::ChangesFlags> transfers);
+    void slotGroupsChanged(QMap<TransferGroupHandler*, TransferGroup::ChangesFlags> groups);
 
 private:
     void exportTransfers(bool plain=false);
@@ -131,11 +128,11 @@ private:
 
     // one-time functions
     void setupActions();
+    
+    KGet * m_kget;
 
     // internal widgets
     ViewsContainer * m_viewsContainer;
-    // dbus modelObserver to export the transfer percents
-    DBusModelObserver *m_dbusModelObserver;
 
     // separated widgets
     DropTarget    * m_drop;
@@ -152,65 +149,13 @@ private:
     QTimer *clipboardTimer;
 
     bool m_startWithoutAnimation;
+    bool m_doTesting;               // UnitTest flag
 
     HttpServer *m_webinterface;
-};
 
-/**
- * Checks every transfer for a percent change to update the mainwindow title
- */
-class MainWindowTransferObserver : public QObject, public TransferObserver
-{
-    Q_OBJECT
-    public:
-        MainWindowTransferObserver(MainWindow *window);
-        virtual ~MainWindowTransferObserver(){}
-
-        virtual void transferChangedEvent(TransferHandler * transfer);
-
-//        virtual void deleteEvent(TransferHandler * transfer);
-
-    private:
-        MainWindow *m_window;
-};
-
-/**
-* Used to update the mainwindow caption when the groups percents change
-*/
-class MainWindowGroupObserver : public QObject, public TransferGroupObserver
-{
-    Q_OBJECT
-    public:
-        MainWindowGroupObserver(MainWindow *window);
-        virtual ~MainWindowGroupObserver() {}
-
-        virtual void groupChangedEvent(TransferGroupHandler * group);
-
-        virtual void addedTransferEvent(TransferHandler * transfer, TransferHandler * after);
-
-        virtual void removedTransferEvent(TransferHandler * transfer);
-/**
-        virtual void movedTransferEvent(TransferHandler * transfer, TransferHandler * after);**/
-
-    private:
-        MainWindow *m_window;
-        MainWindowTransferObserver *m_transferObserver;
-};
-
-class MainWindowModelObserver : public QObject, public ModelObserver
-{
-    Q_OBJECT
-    public:
-        MainWindowModelObserver(MainWindow *window);
-        virtual ~MainWindowModelObserver (){}
-
-        virtual void addedTransferGroupEvent(TransferGroupHandler * group);
-
-        virtual void removedTransferGroupEvent(TransferGroupHandler * group);
-
-    private:
-        MainWindow *m_window;
-        MainWindowGroupObserver *m_groupObserver;
+#ifdef HAVE_QCA2
+    QCA::Initializer m_qcaInit;
+#endif //HAVE_QCA2
 };
 
 #endif
