@@ -2,7 +2,7 @@
 
    Copyright (C) 2005 Dario Massarin <nekkar@libero.it>
    Copyright (C) 2007 by Javier Goday <jgoday@gmail.com>
-   Copyright (C) 2008 by Lukas Appelhans <l.appelhans@gmx.de>
+   Copyright (C) 2008 - 2009 by Lukas Appelhans <l.appelhans@gmx.de>
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
@@ -15,12 +15,14 @@
 #include "core/kget.h"
 #include "core/transfertreemodel.h"
 #include "core/transfergrouphandler.h"
+#include "core/plugin/transferfactory.h"
 #include "settings.h"
 
 #include <QWidget>
 #include <QApplication>
 #include <QClipboard>
 #include <QDir>
+#include <QFileInfo>
 
 #include <KLocale>
 #include <KListWidget>
@@ -41,6 +43,16 @@ public:
         listWidget = 0;
         urlRequester = 0;
     }
+    
+    bool isEmpty()
+    {
+        if (urlRequester) {
+            return urlRequester->text().trimmed().isEmpty();
+        } else if (listWidget) {
+            return !listWidget->count();
+        }
+        return true;
+    }
 
     KUrl::List sources() const
     {
@@ -49,11 +61,11 @@ public:
         {
             for (int i = 0; i != listWidget->count(); i++) {
                 if (listWidget->item(i)->checkState() == Qt::Checked)
-                    list << KUrl(listWidget->item(i)->text());
+                    list << KUrl(listWidget->item(i)->text().trimmed());
             }
         }
         else
-            list << KUrl(urlRequester->text());
+            list << KUrl(urlRequester->text().trimmed());
         return list;
     }
 
@@ -97,6 +109,7 @@ public:
             if (!urlRequester) {
                 urlRequester = new KLineEdit();
                 urlRequester->setClearButtonShown(true);
+                connect(urlRequester, SIGNAL(textChanged(QString)), parent(), SLOT(urlChanged(QString)));
                 m_gridLayout1->addWidget(urlRequester, 0, 1, 1, 1);
                 m_destRequester->setMode(KFile::File);
             }
@@ -155,7 +168,7 @@ public:
             filename = KUrl(filename).directory();
         }
         else if (urlRequester) {
-            filename = KUrl(urlRequester->text()).fileName();
+            filename = KUrl(urlRequester->text().trimmed()).fileName();
         }
 
         for (int i=0;i < list.count();i++)
@@ -164,8 +177,10 @@ public:
                 m_list[i].append('/');
             m_list[i].append(filename);
         }
-        if (!KGlobalSettings::downloadPath().isEmpty()) {
-            m_list << KGlobalSettings::downloadPath();
+
+        const QString downloadPath = KGet::generalDestDir();
+        if (!downloadPath.isEmpty()) {
+            m_list << downloadPath;
         }
         kDebug(5001) << m_list;
 
@@ -182,7 +197,7 @@ public:
                 m_destRequester->comboBox()->addItem(current->defaultFolder());
             m_destRequester->comboBox()->setCurrentIndex(m_destRequester->comboBox()->findText(current->defaultFolder()));
         } else if (current) {
-            m_destRequester->comboBox()->setCurrentIndex(m_destRequester->comboBox()->findText(KGlobalSettings::downloadPath()));
+            m_destRequester->comboBox()->setCurrentIndex(m_destRequester->comboBox()->findText(downloadPath));
         }
     }
 
@@ -209,7 +224,10 @@ public:
         m_destRequester->clear();
         m_displayed = false;
         m_groupComboBox->clear();
-        KGet::addTransferView(m_groupComboBox);
+        //KGet::addTransferView(m_groupComboBox);
+        m_groupComboBox->clear();
+        foreach (TransferGroupHandler * group, KGet::allTransferGroups())
+            m_groupComboBox->addItem(KIcon(group->iconName()), group->name());
         m_groupComboBox->setCurrentIndex(0);
     }
 
@@ -227,7 +245,7 @@ public:
     bool m_displayed; // determines whenever the dialog is already displayed or not (to add new sources)
 };
 
-K_GLOBAL_STATIC(NewTransferDialog, globalInstance)
+NewTransferDialog * m_newTransferDialog = 0;
 
 NewTransferDialog::NewTransferDialog(QWidget *parent) : KDialog(parent),
     m_window(0), m_sources()
@@ -256,47 +274,68 @@ NewTransferDialog::NewTransferDialog(QWidget *parent) : KDialog(parent),
     d->clear();
 
     connect(d->m_groupComboBox, SIGNAL(currentIndexChanged(int)), SLOT(setDefaultDestination()));
-
-    qAddPostRoutine(globalInstance.destroy);
 }
 
 NewTransferDialog::~NewTransferDialog()
 {
-    delete d;
-    qRemovePostRoutine(globalInstance.destroy);
 }
 
-NewTransferDialog *NewTransferDialog::instance(QWidget *parent)
+void NewTransferDialog::del()
 {
-    globalInstance->m_window = parent;
-
-    return globalInstance;
+    m_newTransferDialog->deleteLater();
 }
 
-void NewTransferDialog::showDialog(const QString &srcUrl)
+void NewTransferDialog::showNewTransferDialog(const KUrl &url, QWidget * parent)
 {
-    /*QList<TransferGroupHandler*> groups = KGet::groupsFromExceptions(KUrl(srcUrl));
-    if (!Settings::directoriesAsSuggestion && !groups.isEmpty()) {
-        KGet::addTransfer(KUrl(srcUrl), groups.first()->defaultFolder(), groups.first()->name());
-    }*/
-    KUrl::List list;
-    list << KUrl(srcUrl);
-
-    showDialog(list);
+    showNewTransferDialog(url.isEmpty() ? KUrl::List() : KUrl::List() << url, parent);
 }
 
-void NewTransferDialog::showDialog(const KUrl::List &list)
+void NewTransferDialog::showNewTransferDialog(const KUrl::List &list, QWidget * parent)
+{
+    QString suggestedFileName;
+    if (list.count() == 2)
+    {
+        if (list.at(1).protocol().isEmpty())//When we have no protocol in the second filename, then it's the filename (thrown by konqueror)
+            suggestedFileName = list.at(1).pathOrUrl();
+    }
+    if ((!m_newTransferDialog || m_newTransferDialog->isEmpty()) && list.count() == 1)
+    {
+        KUrl url = list.first();
+        KDialog * dialog = 0;
+        foreach (TransferFactory * factory, KGet::factories()) {
+            dialog = factory->createNewTransferDialog(url, suggestedFileName, !KGet::groupsFromExceptions(url).isEmpty() ? KGet::groupsFromExceptions(url).first() : 0);
+            if (dialog)
+            {
+                if(parent) {
+                    KWindowInfo info = KWindowSystem::windowInfo(parent->winId(), NET::WMDesktop, NET::WMDesktop);
+                    KWindowSystem::setCurrentDesktop(info.desktop());
+                    KWindowSystem::forceActiveWindow(parent->winId());
+                }
+                dialog->exec();
+                return;
+            }
+        }
+    }
+
+    if (!m_newTransferDialog)
+        m_newTransferDialog = new NewTransferDialog(parent);
+
+    m_newTransferDialog->m_window = parent;
+    m_newTransferDialog->showDialog(list, suggestedFileName);
+}
+
+void NewTransferDialog::showDialog(const KUrl::List &list, const QString &suggestedFileName)
 {
     KUrl::List cleanedList;
     kDebug() << "DIRECTORIES AS SUGGESTION" << Settings::directoriesAsSuggestion();
-    if (!Settings::directoriesAsSuggestion())
+    if (!Settings::directoriesAsSuggestion() && !list.isEmpty())
     {
         kDebug(5001) << "No, Directories not as suggestion";
         foreach (const KUrl &url, list)
         {
             QList<TransferGroupHandler*> groups = KGet::groupsFromExceptions(url);
             if (!groups.isEmpty())
-                KGet::addTransfer(url, groups.first()->defaultFolder(), "", groups.first()->name());
+                KGet::addTransfer(url, groups.first()->defaultFolder(), url.fileName(), groups.first()->name());
             else
                 cleanedList << url;
         }
@@ -306,22 +345,19 @@ void NewTransferDialog::showDialog(const KUrl::List &list)
     }
     else
         cleanedList << list;
+
     m_sources << cleanedList;
 
     d->clear();//Let's clear the old stuff
-
     kDebug(5001) << "SET SOURCES " << list << " MULTIPLE " << (m_sources.size () > 1);
     d->setMultiple(m_sources.size() > 1);
 
-    if (list.count() == 2)
-    {
-        if (list.at(1).protocol().isEmpty())//When we have no protocol in the second filename, then it's the filename (thrown by konqueror)
-        {
-            d->setDestinationFileName(list.at(1).pathOrUrl());//We set it to the correct filename
-        }
-    }
+    if (!m_sources.isEmpty()) {
+        if (list.count() == 1 && !suggestedFileName.isEmpty())
+            d->setDestinationFileName(suggestedFileName);
 
-    d->setSource(m_sources);
+        d->setSource(m_sources);
+    }
 
     resizeDialog();
     prepareDialog();
@@ -329,15 +365,17 @@ void NewTransferDialog::showDialog(const KUrl::List &list)
 
 void NewTransferDialog::setDefaultDestination()
 {
-    if (m_sources.isEmpty()) {
-        return;
-    }
+    //if (m_sources.isEmpty()) {
+    //    return;
+    //}
     QStringList list;
     foreach (TransferGroupHandler *handler, KGet::allTransferGroups()) {
         if (!handler->defaultFolder().isEmpty())
             list << handler->defaultFolder();
     }
     d->setDestination(m_sources, list);
+    if (!m_sources.isEmpty())
+        urlChanged(m_sources.first().path());
     //d->setDestination(m_sources, KGet::defaultFolders(m_sources.first().path(), d->transferGroup()));
 }
 
@@ -352,6 +390,7 @@ void NewTransferDialog::prepareDialog()
     }
 
     if (!d->m_displayed) {
+        kDebug(5001) << "Exec the dialog!";
         d->m_displayed = true;
 
         KDialog::exec();
@@ -361,7 +400,7 @@ void NewTransferDialog::prepareDialog()
             QString destDir = d->destination();
             m_sources = d->sources();
 
-            destDir = KUrl(destDir).path();
+            destDir = KUrl(destDir).toLocalFile();
 
             QString dir;
             if (QFileInfo(destDir).isDir())
@@ -380,6 +419,7 @@ void NewTransferDialog::prepareDialog()
         }
 
         m_sources.clear();
+        d->clear();
     }
 }
 
@@ -392,4 +432,22 @@ void NewTransferDialog::resizeDialog()
     }
 }
 
+bool NewTransferDialog::isEmpty()
+{
+    return d->isEmpty();
+}
+
+void NewTransferDialog::urlChanged(const QString &text)
+{
+    if (d->m_multiple)
+        return;
+    KUrl url(text.trimmed());
+    //if (d->m_destRequester->url()->isEmpty())
+    //    d->setDestination(m_sources, QStringList());
+    if (QFileInfo(d->m_destRequester->url().toLocalFile()).isDir())
+        d->setDestinationFileName(url.fileName());
+
+    kDebug() << url << url.fileName() << d->m_destRequester->url().fileName();
+}
+
 #include "newtransferdialog.moc"
