@@ -39,6 +39,7 @@
 #include <kicon.h>
 #include <kaction.h>
 #include <kmenu.h>
+#include <kcolordialog.h>
 
 #include "addaccountwizard.h"
 #include "editaccountwidget.h"
@@ -52,28 +53,6 @@
 #include "identitydialog.h"
 
 
-
-class KopeteAccountLVI : public QTreeWidgetItem
-{
-	public:
-		KopeteAccountLVI( Kopete::Account *a, QTreeWidgetItem* parent) : QTreeWidgetItem(parent) , m_account(a) { }
-		Kopete::Account *account() { return m_account; }
-
-	private:
-		//need to be guarded because some accounts may be linked (that's the case of jabber transports)
-		QPointer<Kopete::Account> m_account;
-};
-
-class KopeteIdentityLVI : public QTreeWidgetItem
-{
-	public:
-		KopeteIdentityLVI( Kopete::Identity *i, QTreeWidget* parent) : QTreeWidgetItem(parent), m_identity (i) { }
-		Kopete::Identity *identity() { return m_identity; }
-
-	private:
-		Kopete::Identity *m_identity;
-};
-
 K_PLUGIN_FACTORY( KopeteAccountConfigFactory,
 		registerPlugin<KopeteAccountConfig>(); )
 K_EXPORT_PLUGIN( KopeteAccountConfigFactory("kcm_kopete_accountconfig") )
@@ -91,6 +70,7 @@ KopeteAccountConfig::KopeteAccountConfig( QWidget *parent, const QVariantList &a
 	configureActions();
 	configureMenus();
 
+	connect( mAccountList,  SIGNAL(itemPositionChanged()), this, SLOT(changed()) );
 	connect( mAccountList,  SIGNAL( itemSelectionChanged() ), this, SLOT( slotItemSelected() ) );
 	connect( mAccountList,  SIGNAL( itemDoubleClicked(QTreeWidgetItem*, int) ), this, SLOT( slotModify() ) );
 	connect( mAccountList,  SIGNAL( itemChanged ( QTreeWidgetItem * , int )), this, SLOT( slotItemChanged(QTreeWidgetItem*) ) );
@@ -98,6 +78,9 @@ KopeteAccountConfig::KopeteAccountConfig( QWidget *parent, const QVariantList &a
 
 	// this ensures that newly created accounts are assigned to the selected identity
 	connect( Kopete::AccountManager::self(), SIGNAL(accountRegistered(Kopete::Account *)), this, SLOT(slotAccountAdded(Kopete::Account *)) );
+    connect( Kopete::AccountManager::self(), SIGNAL(accountUnregistered(const Kopete::Account*)), this, SLOT(slotAccountRemoved(const Kopete::Account *)) );
+
+	mAccountList->installEventFilter( this );
 
 	setButtons( Help );
 	load();
@@ -121,17 +104,20 @@ KopeteIdentityLVI* KopeteAccountConfig::selectedIdentity()
 
 void KopeteAccountConfig::save()
 {
-	// The priority is disabled because of identities and because there is no way to edit it anymore
-	/*uint priority = mAccountList->topLevelItemCount();
-	
-	KopeteAccountLVI *i;
-	for( int j=0; j<mAccountList->topLevelItemCount(); ++j )
+	uint priority = 0;
+	for ( int i = 0; i < mAccountList->topLevelItemCount(); i++ )
+		priority += mAccountList->topLevelItem( i )->childCount();
+			
+	for ( int i = 0; i < mAccountList->topLevelItemCount(); i++ )
 	{
-		i = static_cast<KopeteAccountLVI*>( mAccountList->topLevelItem( j ) );
-		if(!i->account())
-			continue;
-		i->account()->setPriority( priority-- );
-	}*/
+		KopeteIdentityLVI* identity = dynamic_cast<KopeteIdentityLVI*>( mAccountList->topLevelItem( i ) );
+		for ( int j = 0; j < identity->childCount(); j++ )
+		{
+			KopeteAccountLVI* account = dynamic_cast<KopeteAccountLVI*>( identity->child( j ) );
+			account->account()->setIdentity( identity->identity() );
+			account->account()->setPriority( priority-- );
+		}
+	}
 
 	Kopete::AccountManager::self()->save();
 	Kopete::IdentityManager::self()->save();
@@ -140,16 +126,34 @@ void KopeteAccountConfig::save()
 	//load(); //refresh the colred accounts (in case of apply)
 }
 
+bool accountPriorityLessThan(const Kopete::Account* a, const Kopete::Account* b)
+{
+	return (a->priority() > b->priority());
+}
+
+bool identityPriorityLessThan(const Kopete::Identity* a, const Kopete::Identity* b)
+{
+	if ( a->accounts().isEmpty() )
+		return false;
+	
+	if ( b->accounts().isEmpty() && !a->accounts().isEmpty() )
+		return true;
+	
+	return (a->accounts().first()->priority() > b->accounts().first()->priority());
+}
+
 void KopeteAccountConfig::load()
 {
 	mAccountList->clear();
 
-	//FIXME: this doesn't work
-	mAccountList->invisibleRootItem()->setFlags(mAccountList->invisibleRootItem()->flags() & ~Qt::ItemIsDropEnabled);
-
 	QHash<Kopete::Identity *,QTreeWidgetItem *> identityItemHash;
 	Kopete::Identity *defaultIdentity = Kopete::IdentityManager::self()->defaultIdentity();
-	foreach(Kopete::Identity *i, Kopete::IdentityManager::self()->identities())
+
+	// Sort by priority, the priority is take from identity accounts because identity doesn't have priority
+	QList<Kopete::Identity*> identityList = Kopete::IdentityManager::self()->identities();
+	qSort( identityList.begin(), identityList.end(), identityPriorityLessThan );
+
+	foreach(Kopete::Identity *i, identityList)
 	{
 		//KopeteIdentityLVI *lvi = new KopeteIdentityLVI( i, mIdentityList );
 		QTreeWidgetItem *identityItem = new KopeteIdentityLVI( i, mAccountList );
@@ -160,8 +164,6 @@ void KopeteAccountConfig::load()
 		
 		identityItem->setExpanded( true );
 		
-		identityItem->setFlags( identityItem->flags() & ~Qt::ItemIsDragEnabled );
-		
 		if (i == defaultIdentity)
 		{
 			QFont font = identityItem->font( 0 );
@@ -174,31 +176,36 @@ void KopeteAccountConfig::load()
 		identityItemHash.insert(i,identityItem);
 	}
 
-	foreach(Kopete::Account *i , Kopete::AccountManager::self()->accounts() )
+	// Sort by priority
+	QList<Kopete::Account*> accountList = Kopete::AccountManager::self()->accounts();
+	qSort( accountList.begin(), accountList.end(), accountPriorityLessThan );
+
+	foreach( Kopete::Account *account, accountList )
 	{
-		Kopete::Identity *idnt = i->identity();
+		Kopete::Identity *idnt = account->identity();
 		
 		Q_ASSERT(identityItemHash.contains(idnt));
-		KopeteAccountLVI *lvi = new KopeteAccountLVI( i, identityItemHash[idnt] );
-		lvi->setText( 0, i->accountLabel() );
-		lvi->setIcon( 0, i->myself()->onlineStatus().iconFor( i) );
+		KopeteAccountLVI *lvi = new KopeteAccountLVI( account, identityItemHash[idnt] );
+		lvi->setText( 0, account->accountLabel() );
+		lvi->setIcon( 0, account->myself()->onlineStatus().iconFor( account ) );
 		QFont font = lvi->font( 0 );
 		font.setBold( true );
 		lvi->setFont( 0, font );
 
 		lvi->setSizeHint( 0, QSize(0, 42) );
 		
-		lvi->setText( 1, i->myself()->onlineStatus().statusTypeToString(i->myself()->onlineStatus().status()) );
+		lvi->setText( 1, account->myself()->onlineStatus().statusTypeToString(account->myself()->onlineStatus().status()) );
 		lvi->setTextAlignment( 1, Qt::AlignRight | Qt::AlignVCenter );
 		lvi->setFont( 1, font );
 	
 		lvi->setFlags( (lvi->flags() & ~Qt::ItemIsDropEnabled) | Qt::ItemIsUserCheckable );
-		lvi->setCheckState ( 0, i->excludeConnect() ? Qt::Unchecked : Qt::Checked );
+		lvi->setCheckState ( 0, account->excludeConnect() ? Qt::Unchecked : Qt::Checked );
 
-		connect( i->myself(), SIGNAL(onlineStatusChanged(Kopete::Contact*, const Kopete::OnlineStatus&, const Kopete::OnlineStatus&)),
+		connect( account->myself(), SIGNAL(onlineStatusChanged(Kopete::Contact*, const Kopete::OnlineStatus&, const Kopete::OnlineStatus&)),
 				 this, SLOT(slotOnlineStatusChanged(Kopete::Contact*, const Kopete::OnlineStatus&, const Kopete::OnlineStatus&)));
 	}
 
+	changed( false );
 	slotItemSelected();
 }
 
@@ -210,6 +217,7 @@ void KopeteAccountConfig::slotItemSelected()
 	mButtonAccountRemove->setEnabled( accountSelected );
 	m_actionAccountSwitchIdentity->setEnabled( accountSelected && hasMultipleIdentities );
 	mButtonAccountSwitchIdentity->setEnabled( m_actionAccountSwitchIdentity->isEnabled() );
+	mButtonAccountSetColor->setEnabled( accountSelected );
 
 	bool identitySelected = selectedIdentity();
 	bool isDefaultIdentity = (identitySelected && Kopete::IdentityManager::self()->defaultIdentity() == selectedIdentity()->identity());
@@ -308,7 +316,6 @@ void KopeteAccountConfig::removeAccount()
 					QString(), KMessageBox::Notify | KMessageBox::Dangerous ) == KMessageBox::Continue )
 		{
 			Kopete::AccountManager::self()->removeAccount( i );
-			delete lvi;
 		}
 	}
 }
@@ -372,6 +379,24 @@ void KopeteAccountConfig::slotAccountSwitchIdentity()
 	load();
 }
 
+void KopeteAccountConfig::slotAccountSetColor()
+{
+	KopeteAccountLVI *lvi = selectedAccount();
+
+	if ( !lvi || !lvi->account() )
+		return;
+
+	Kopete::Account *a = lvi->account();
+
+	QColor color = a->color();
+
+	if ( KColorDialog::getColor(color, Qt::black, this) == KColorDialog::Accepted ) {
+		a->setColor(color);
+	}
+
+	load();
+}
+
 void KopeteAccountConfig::slotSetDefaultIdentity()
 {
 	KopeteIdentityLVI *lvi = selectedIdentity();
@@ -459,6 +484,21 @@ void KopeteAccountConfig::slotAccountAdded( Kopete::Account * account )
 	load();
 }
 
+void KopeteAccountConfig::slotAccountRemoved( const Kopete::Account * account )
+{
+	QList<QTreeWidgetItem*> items = mAccountList->findItems("", Qt::MatchContains | Qt::MatchRecursive);
+	QList<QTreeWidgetItem*>::iterator it;
+	for (it = items.begin(); it != items.end(); ++it)
+	{
+		KopeteAccountLVI *lvi = dynamic_cast<KopeteAccountLVI*>(*it);
+		if ( lvi && lvi->account() == account)
+		{
+			delete lvi;
+			break;
+		}
+	}
+}
+
 void KopeteAccountConfig::slotItemChanged(QTreeWidgetItem* item)
 {
 	if(!item)
@@ -475,19 +515,26 @@ void KopeteAccountConfig::slotItemChanged(QTreeWidgetItem* item)
 	}
 }
 
-void KopeteAccountConfig::contextMenuEvent ( QContextMenuEvent * event )
-{	
-	KopeteIdentityLVI *ilvi = selectedIdentity();
-	if ( ilvi && ilvi->identity() )
+bool KopeteAccountConfig::eventFilter( QObject *obj, QEvent *event )
+{
+	if ( obj == mAccountList && event->type() == QEvent::ContextMenu )
 	{
-		m_identityContextMenu->popup(event->globalPos());
+		QContextMenuEvent *cmEvent = static_cast<QContextMenuEvent *>(event);
+		KopeteIdentityLVI *ilvi = selectedIdentity();
+		if ( ilvi && ilvi->identity() )
+		{
+			m_identityContextMenu->popup(cmEvent->globalPos());
+		}
+
+		KopeteAccountLVI *alvi = selectedAccount();
+		if ( alvi && alvi->account() )
+		{
+			m_accountContextMenu->popup(cmEvent->globalPos());
+		}
+		return true;
 	}
 
-	KopeteAccountLVI *alvi = selectedAccount();
-	if ( alvi && alvi->account() )
-	{
-		m_accountContextMenu->popup(event->globalPos());
-	}
+	return QObject::eventFilter( obj, event );
 }
 
 void KopeteAccountConfig::configureActions()
@@ -523,6 +570,12 @@ void KopeteAccountConfig::configureActions()
 	connect( m_actionAccountSwitchIdentity, SIGNAL(triggered(bool)), this, SLOT(slotAccountSwitchIdentity()) );
 	connect( mButtonAccountSwitchIdentity, SIGNAL(clicked()), m_actionAccountSwitchIdentity, SLOT(trigger()) );
 
+	// Set/clear custom color for account
+	m_actionAccountSetColor = new KAction( i18n( "Set C&olor..." ), this );
+	mButtonAccountSetColor->setText( m_actionAccountSetColor->text() );
+	connect( m_actionAccountSetColor, SIGNAL(triggered(bool)), this, SLOT(slotAccountSetColor()) );
+	connect( mButtonAccountSetColor, SIGNAL(clicked()), m_actionAccountSetColor, SLOT(trigger()) );
+
 	// Add identity
 	m_actionIdentityAdd = new KAction( i18n( "Add &Identity..." ), this );
 	m_actionIdentityAdd->setIcon( KIcon("list-add") );
@@ -568,6 +621,7 @@ void KopeteAccountConfig::configureMenus()
 	m_accountContextMenu = new KMenu ( this );
 	m_accountContextMenu->addAction( m_actionAccountModify );
 	m_accountContextMenu->addAction( m_actionAccountRemove );
+	m_accountContextMenu->addAction( m_actionAccountSetColor );
 
 	// Identity management context menu
 	m_identityContextMenu = new KMenu ( this );
