@@ -70,15 +70,19 @@ AVDeviceConfig::AVDeviceConfig(QWidget *parent, const QVariantList &args)
 	connect(mPrfsVideoDevice->mImageAsMirror,                SIGNAL(toggled(bool)),     this, SLOT(slotImageAsMirrorChanged(bool)));
 
 	mVideoDevicePool = Kopete::AV::VideoDevicePool::self();
-	mVideoDevicePool->open();
-	mVideoDevicePool->setSize(320, 240);
+
+	if (EXIT_SUCCESS == mVideoDevicePool->open())
+	{
+		setVideoInputParameters();
+		startCapturing();
+	}
 
 	mVideoDevicePool->fillDeviceKComboBox(mPrfsVideoDevice->mDeviceKComboBox);
 	mVideoDevicePool->fillInputKComboBox(mPrfsVideoDevice->mInputKComboBox);
 	mVideoDevicePool->fillStandardKComboBox(mPrfsVideoDevice->mStandardKComboBox);
-	setVideoInputParameters();
 
-	mVideoDevicePool->startCapturing();
+	mPrfsVideoDevice->VideoTabWidget->setTabEnabled(1, mVideoDevicePool->size());
+	mPrfsVideoDevice->VideoTabWidget->setTabEnabled(2, mVideoDevicePool->size());
 
 	connect(mVideoDevicePool, SIGNAL(deviceRegistered(const QString &) ),
 			SLOT(deviceRegistered(const QString &)) );
@@ -86,13 +90,6 @@ AVDeviceConfig::AVDeviceConfig(QWidget *parent, const QVariantList &args)
 			SLOT(deviceUnregistered(const QString &)) );
 
 	connect(&qtimer, SIGNAL(timeout()), this, SLOT(slotUpdateImage()) );
-#define DONT_TRY_TO_GRAB 1
-#if DONT_TRY_TO_GRAB
-	if ( mVideoDevicePool->hasDevices() ) {
-		qtimer.start(40);
-		mPrfsVideoDevice->mVideoImageLabel->setScaledContents(true);
-	}
-#endif
 }
 
 
@@ -152,12 +149,13 @@ void AVDeviceConfig::slotDeviceKComboBoxChanged(int){
 	kDebug() << "kopete:config (avdevice): slotDeviceKComboBoxChanged(int) Current device: " << mVideoDevicePool->currentDevice() << "New device: " << newdevice;
 	if ((newdevice>=0 && newdevice < mVideoDevicePool->m_videodevice.size())&&(newdevice!=mVideoDevicePool->currentDevice()))
 	{
-	kDebug() << "kopete:config (avdevice): slotDeviceKComboBoxChanged(int) should change device. ";
+		kDebug() << "kopete:config (avdevice): slotDeviceKComboBoxChanged(int) should change device. ";
+		stopCapturing();
+		mVideoDevicePool->close();
 		mVideoDevicePool->open(newdevice);
-		mVideoDevicePool->setSize(320, 240);
 		mVideoDevicePool->fillInputKComboBox(mPrfsVideoDevice->mInputKComboBox);
-		mVideoDevicePool->startCapturing();
 		setVideoInputParameters();
+		startCapturing();
 		kDebug() << "kopete:config (avdevice): slotDeviceKComboBoxChanged(int) called. ";
 		emit changed( true );
 	}
@@ -165,12 +163,14 @@ void AVDeviceConfig::slotDeviceKComboBoxChanged(int){
 
 void AVDeviceConfig::slotInputKComboBoxChanged(int){
 	int newinput = mPrfsVideoDevice->mInputKComboBox->currentIndex();
-	if((newinput < mVideoDevicePool->inputs()) && ( newinput !=mVideoDevicePool->currentInput()))
+	if ((newinput < mVideoDevicePool->inputs()) && (newinput != mVideoDevicePool->currentInput()))
 	{
-		mVideoDevicePool->selectInput(mPrfsVideoDevice->mInputKComboBox->currentIndex());
-		mVideoDevicePool->fillStandardKComboBox(mPrfsVideoDevice->mStandardKComboBox);
+		stopCapturing();
+ 		mVideoDevicePool->selectInput(mPrfsVideoDevice->mInputKComboBox->currentIndex());
+ 		mVideoDevicePool->fillStandardKComboBox(mPrfsVideoDevice->mStandardKComboBox);
 		setVideoInputParameters();
-		emit changed( true );
+		startCapturing();
+ 		emit changed( true );
 	}
 }
 
@@ -243,21 +243,61 @@ void AVDeviceConfig::deviceRegistered( const QString & udi )
 	mVideoDevicePool->fillInputKComboBox(mPrfsVideoDevice->mInputKComboBox);
 	mVideoDevicePool->fillStandardKComboBox(mPrfsVideoDevice->mStandardKComboBox);
 
-	// update the mVideoImageLabel to show the camera frames
-	mVideoDevicePool->open();
+	mPrfsVideoDevice->VideoTabWidget->setTabEnabled(1, mVideoDevicePool->size());
+	mPrfsVideoDevice->VideoTabWidget->setTabEnabled(2, mVideoDevicePool->size());
+
+	if (mVideoDevicePool->size() < 2) // otherwise we are already capturing
+	{
+		if (EXIT_SUCCESS == mVideoDevicePool->open())
+		{
+			setVideoInputParameters();
+			startCapturing();
+		}
+	}
+}
+
+void AVDeviceConfig::deviceUnregistered( const QString & udi )
+{
+	mVideoDevicePool->fillDeviceKComboBox(mPrfsVideoDevice->mDeviceKComboBox);
+	mVideoDevicePool->fillInputKComboBox(mPrfsVideoDevice->mInputKComboBox);
+	mVideoDevicePool->fillStandardKComboBox(mPrfsVideoDevice->mStandardKComboBox);
+
+	mPrfsVideoDevice->VideoTabWidget->setTabEnabled(1, mVideoDevicePool->size());
+	mPrfsVideoDevice->VideoTabWidget->setTabEnabled(2, mVideoDevicePool->size());
+
+	if (capturingDevice_udi == udi)
+	{
+		qtimer.stop();
+		mPrfsVideoDevice->mVideoImageLabel->setScaledContents(false);
+		mPrfsVideoDevice->mVideoImageLabel->setPixmap(KIcon("camera-web").pixmap(128,128));
+		capturingDevice_udi.clear();
+
+		if (mVideoDevicePool->size())
+		{
+			if (EXIT_SUCCESS == mVideoDevicePool->open())
+			{
+				setVideoInputParameters();
+				startCapturing();
+			}
+		}
+	}
+}
+
+void AVDeviceConfig::startCapturing()
+{
 	mVideoDevicePool->setSize(320, 240);
 	mVideoDevicePool->startCapturing();
-
-	setVideoInputParameters();
-
+	capturingDevice_udi = mVideoDevicePool->currentDeviceUdi();
 	qtimer.start(40);
 	mPrfsVideoDevice->mVideoImageLabel->setScaledContents(true);
 }
 
-
-void AVDeviceConfig::deviceUnregistered( const QString & udi )
+void AVDeviceConfig::stopCapturing()
 {
-/*	mVideoDevicePool->fillDeviceKComboBox(mPrfsVideoDevice->mDeviceKComboBox);
-	mVideoDevicePool->fillInputKComboBox(mPrfsVideoDevice->mInputKComboBox);
-	mVideoDevicePool->fillStandardKComboBox(mPrfsVideoDevice->mStandardKComboBox);
-*/}
+	qtimer.stop();
+	mVideoDevicePool->stopCapturing();
+	mPrfsVideoDevice->mVideoImageLabel->setScaledContents(false);
+	mPrfsVideoDevice->mVideoImageLabel->setPixmap(KIcon("camera-web").pixmap(128,128));
+	capturingDevice_udi.clear();
+}
+
