@@ -239,7 +239,7 @@ void KopeteApplication::slotAllPluginsLoaded()
 	bool found = false;
 
 	//find first Status for OnlineStatus
-	for ( QList <Kopete::Status::StatusItem *>::ConstIterator it = statusList.begin(); it != statusList.end(); ++it ) {
+	for ( QList <Kopete::Status::StatusItem *>::ConstIterator it = statusList.constBegin(); it != statusList.constEnd(); ++it ) {
 		if ( ! (*it)->isGroup() && (*it)->category() == setStatus ) {
 			title = (*it)->title();
 			message = (static_cast <Kopete::Status::Status*> (*it))->message(); //if it is not group, it status
