@@ -458,6 +458,11 @@ QString MetaContact::statusIcon() const
 				return icon( ContactListElement::Away );
 			else
 				return QString::fromUtf8( "user-away" );
+		case OnlineStatus::Busy:
+			if( useCustomIcon() )
+				return icon( ContactListElement::Away ); //Might want to create custom for busy too
+			else
+				return QString::fromUtf8( "user-busy" );
 
 		case OnlineStatus::Unknown:
 			if( useCustomIcon() )
@@ -484,6 +489,8 @@ QString MetaContact::statusString() const
 			return i18n( "Online" );
 		case OnlineStatus::Away:
 			return i18n( "Away" );
+		case OnlineStatus::Busy:
+			return i18n( "Busy" );
 		case OnlineStatus::Offline:
 			return i18n( "Offline" );
 		case OnlineStatus::Unknown:
@@ -509,6 +516,18 @@ bool MetaContact::isOnline() const
 	return false;
 }
 
+bool MetaContact::isAlwaysVisible() const
+{
+	QListIterator<Contact *> it( d->contacts );
+	while ( it.hasNext() )
+	{
+		Contact* c = it.next();
+		if ( c && c->property( Kopete::Global::Properties::self()->isAlwaysVisible() ).value().toBool() )
+			return true;
+	}
+	return false;
+}
+
 bool MetaContact::isReachable() const
 {
 	if ( isOnline() )
@@ -705,7 +724,7 @@ KUrl MetaContact::customPhoto() const
 void MetaContact::setPhoto( const KUrl &url )
 {
 	d->photoUrl = url;
-	d->customPicture.setPicture(url.path());
+	d->customPicture.setPicture(url.toLocalFile());
 
 	if ( photoSource() == SourceCustom )
 	{
@@ -935,6 +954,11 @@ void MetaContact::slotPropertyChanged( PropertyContainer* _subcontact, const QSt
 
 		emit statusMessageChanged( this );
 	}
+
+	// Here we abuse the onlineStatusChanged signal to force the contact list to refresh and hide the
+	// contact if he is offline and his isAlwaysVisible property changed to false
+	if ( key == Kopete::Global::Properties::self()->isAlwaysVisible().key() && oldValue != newValue )
+		emit onlineStatusChanged( this, d->onlineStatus );
 }
 
 void MetaContact::moveToGroup( Group *from, Group *to )
@@ -1268,9 +1292,7 @@ void MetaContact::onlineStatusNotification( Kopete::Contact * c )
 					t = signedIn;	// contact has gone from offline to something else, it's a sign-in
 				}
 			}
-			else if ( d->notifyOnlineStatus.status() == Kopete::OnlineStatus::Online
-			          || d->notifyOnlineStatus.status() == Kopete::OnlineStatus::Away
-			          || d->notifyOnlineStatus.status() == Kopete::OnlineStatus::Invisible)
+			else if ( d->notifyOnlineStatus.isDefinitelyOnline()) //Is that correct?
 			{
 				if ( newNotifyOnlineStatus.status() == Kopete::OnlineStatus::Offline )
 				{
