@@ -26,6 +26,7 @@
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <qtimer.h>
+#include <QDateTime>
 
 #define PROTOCOL_MAX 8
 #define PROTOCOL_MIN 6
@@ -150,6 +151,12 @@ void Skype::setOnline() {
 	queueSkypeMessage("SET USERSTATUS ONLINE", true);//just send the message
 }
 
+void Skype::setUserProfileRichMoodText(const QString &message) {
+	kDebug(SKYPE_DEBUG_GLOBAL);
+
+	queueSkypeMessage(QString("SET PROFILE RICH_MOOD_TEXT %1").arg(message),true);
+}
+
 void Skype::setOffline() {
 	kDebug(SKYPE_DEBUG_GLOBAL);
 	d->showDeadMessage = false;
@@ -354,6 +361,12 @@ void Skype::skypeMessage(const QString &message) {
 		} else if ( type == "ABOUT" ) {
 			/// TODO: Implement info ABOUT
 			kDebug(SKYPE_DEBUG_GLOBAL) << "Info ABOUT is not implemented for contact, ignored";
+		} else if ( type == "RECEIVEDAUTHREQUEST") {
+			if ( getAuthor(contactId) != Skype::Author ) {// Skype send all authorize request min 2x, filter if contact is authorized
+				const QString &info = message.section(' ', 3);
+				emit receivedAuth(contactId, info);
+				kDebug(SKYPE_DEBUG_GLOBAL) << "Received auth request from" << contactId;
+			}
 		} else {
 			kDebug(SKYPE_DEBUG_GLOBAL) << "Unknown message for contact, ignored";
 		}
@@ -422,6 +435,17 @@ void Skype::skypeMessage(const QString &message) {
 					if (!body.isEmpty())//sometimes skype shows empty messages, just ignore them
 						emit outgoingMessage(body, chat);
 			}
+		} else if ( type == "EDITED_TIMESTAMP" ) {//timestamp of message was edited
+			///TODO: Implement this
+			kDebug(SKYPE_DEBUG_GLOBAL) << "Timestamp of message" << messageId << "was edited, this is not implemented now";
+		} else if ( type == "EDITED_BY" ) {//this message was edited
+			///TODO: Implement this
+			QString editedBy = message.section(' ', 3, 3).trimmed();
+			kDebug(SKYPE_DEBUG_GLOBAL) << "Message" << messageId << "was edited by" << editedBy << ", this is not implemented now";
+		} else if ( type == "BODY" ) { //This message was edited and has new body
+			///TODO: Implement this
+			QString newBody = message.section(' ', 3, 3).trimmed();
+			kDebug(SKYPE_DEBUG_GLOBAL) << "Message" << messageId << "was edited to" << newBody << ", this is not implemented now";
 		}
 	} else if (messageType == "CHATMESSAGES") {
 		if (d->searchFor == "MISSEDMESSAGES") {//Theese are messages we did not read yet
@@ -500,6 +524,15 @@ void Skype::skypeMessage(const QString &message) {
 				emit callError(callId, errorText);
 			}
 			emit callStatus(callId, status);
+		} else if ( message.section(' ', 2, 2).trimmed().toUpper() == "VIDEO_RECEIVE_STATUS" ) {
+			const QString &status = message.section(' ', 3, 3).trimmed().toUpper();
+			if ( status == "RUNNING" ) {
+				kDebug(SKYPE_DEBUG_GLOBAL) << "Start receiving video";
+				emit startReceivingVideo(callId);
+			} else if ( status == "STOPPING" ) {
+				kDebug(SKYPE_DEBUG_GLOBAL) << "Stop receiving video";
+				emit stopReceivingVideo(callId);
+			}
 		}
 	} else if (messageType == "CURRENTUSERHANDLE") {
 		QString user = message.section(' ', 1, 1).trimmed();
@@ -605,19 +638,26 @@ void Skype::hitchHike(const QString &messageId) {
 
 	const QString &chatType = (d->connection % QString("GET CHAT %1 STATUS").arg(chat)).section(' ', 3, 3).trimmed().toUpper();
 
+	bool ok = false;
+	uint utime = (d->connection % QString("GET CHATMESSAGE %1 TIMESTAMP").arg(messageId)).section(' ', 3, 3).trimmed().toUInt(&ok);
+	if ( ! ok )
+		utime = QDateTime::currentDateTime().toTime_t();
+
+	const QDateTime &timeStamp = QDateTime::fromTime_t(utime);
+
 	if ((chatType == "LEGACY_DIALOG") || (chatType == "DIALOG")) {
 
 		const QString &user = (d->connection % QString("GET CHATMESSAGE %1 FROM_HANDLE").arg(messageId)).section(' ', 3, 3).trimmed();//ask skype for a sender of that message and filter out the blouat around (like CHATMESSAGE 123...)
 
 		if ((d->hitch) || (d->account.userHasChat(user))) {//it can be read eather if the hitchhiking non-chat messages is enabled or if the user already has opened a chat
-			emit receivedIM(user, (d->connection % QString("GET CHATMESSAGE %1 BODY").arg(messageId)).section(' ', 3), messageId);//ask skype for the body and filter out the bload, we want only the text and make everyone aware that we received a message
+			emit receivedIM(user, (d->connection % QString("GET CHATMESSAGE %1 BODY").arg(messageId)).section(' ', 3), messageId, timeStamp);//ask skype for the body and filter out the bload, we want only the text and make everyone aware that we received a message
 			if (d->mark) //We should mark it as read
 				d->connection << QString("SET CHATMESSAGE %1 SEEN").arg(messageId);//OK, just tell skype it is read
 		}
 	} else {
 		if ((d->hitch) || (d->account.chatExists(chat))) {
 			const QString &user = (d->connection % QString("GET CHATMESSAGE %1 FROM_HANDLE").arg(messageId)).section(' ', 3, 3).trimmed();
-			emit receivedMultiIM(chat, (d->connection % QString("GET CHATMESSAGE %1 BODY").arg(messageId)).section(' ', 3), messageId, user);
+			emit receivedMultiIM(chat, (d->connection % QString("GET CHATMESSAGE %1 BODY").arg(messageId)).section(' ', 3), messageId, user, timeStamp);
 			if (d->mark)
 				d->connection << QString("SET CHATMESSAGE %1 SEEN").arg(messageId);
 		}
@@ -630,6 +670,10 @@ void Skype::send(const QString &user, const QString &message) {
 	d->connection << QString("MESSAGE %1 %2").arg(user).arg(message);//just ask skype to send it
 }
 
+void Skype::editMessage(int messageId, const QString &newMessage) {
+	d->connection << QString("CHATMESSAGE %1 BODY %2").arg(messageId).arg(newMessage);
+}
+
 void Skype::setScanForUnread(bool value) {
 	d->scanForUnread = value;
 }
@@ -974,4 +1018,27 @@ bool Skype::openFileTransfer(const QString &user, const QString &url) {
 		return false;
 }
 
+QStringList Skype::searchUsers(const QString &string) {
+	kDebug(SKYPE_DEBUG_GLOBAL) << string;
+	return (d->connection % QString("SEARCH USERS %1").arg(string)).section(' ', 1).trimmed().split(' ');
+}
+
+bool Skype::supportVideo(const QString &user) {
+	kDebug(SKYPE_DEBUG_GLOBAL) << user;
+	if ( (d->connection % QString("GET USER %1 IS_VIDEO_CAPABLE").arg(user)).section(' ', 3).trimmed().toUpper() == "TRUE" )
+		return true;
+	else
+		return false;
+}
+
+void Skype::startSendingVideo(const QString &callId) {
+	kDebug(SKYPE_DEBUG_GLOBAL) << callId;
+	d->connection << QString("ALTER CALL %1 START_VIDEO_SEND").arg(callId);
+}
+
+void Skype::stopSendingVideo(const QString &callId) {
+	kDebug(SKYPE_DEBUG_GLOBAL) << callId;
+	d->connection << QString("ALTER CALL %1 STOP_VIDEO_SEND").arg(callId);
+}
+
 #include "skype.moc"
