@@ -24,6 +24,7 @@
 
 class SkypePrivate;
 class SkypeAccount;
+class QDateTime;
 
 /**
  * @author Kopete Developers
@@ -218,12 +219,43 @@ class Skype : public QObject
 		 * @url url of local file
 		 * @return true if transfer opened
 		 */
-		bool openFileTransfer(const QString &user, const QString &url = QString::null);
+		bool openFileTransfer(const QString &user, const QString &url = QString());
+		/**
+		 * Search for skype contacts
+		 * @param string skype user name or email (with "@")
+		 * @return all users for string string
+		 */
+		QStringList searchUsers(const QString &string);
+
+		///Video section
+
+		/**
+		 * Check if user has video support
+		 * @user for check
+		 */
+		bool supportVideo(const QString &user);
+
+		/**
+		 * Start sending video in active call
+		 * @callId id of call
+		 */
+		void startSendingVideo(const QString &callId);
+
+		/**
+		 * Stop sending video in active video call
+		 * @callId id of call
+		 */
+		void stopSendingVideo(const QString &callId);
+
 	public slots:
 		/**
 		 * Tell the skype to go online
 		 */
 		void setOnline();
+		/**
+		 * Sets the rich text mood for the user profile
+		 */
+		void setUserProfileRichMoodText(const QString &);
 		/**
 		 * Tell the skype to go offline
 		 */
@@ -270,6 +302,12 @@ class Skype : public QObject
 		 * @param body What to send
 		 */
 		void send(const QString &user, const QString &body);
+		/**
+		 * Edit sent message
+		 * @param messageId Id of sent message
+		 * @parma newMessage Body of new message
+		 */
+		void editMessage(int messageId, const QString &newMessage);
 		/**
 		 * Send a message to a given chat
 		 * @param chat What chat to send it in
@@ -435,16 +473,18 @@ class Skype : public QObject
 		 * @param user Contact ID of user that sent it. It is NOT guaranteed that the user is in list!
 		 * @param body The message body that was received
 		 * @param messageId ID of that message
+		 * @param timeStamp time when message was send
 		 */
-		void receivedIM(const QString &user, const QString &body, const QString &messageId);
+		void receivedIM(const QString &user, const QString &body, const QString &messageId, const QDateTime &timeStamp);
 		/**
 		 * This is emitted when a new message from multi-user chat is received
 		 * @param chat Id of the chat
 		 * @param body Tect of the message
 		 * @param messageId Id of this message to get information about it if needed
 		 * @param user Who sent it to that chat (ID)
+		 * @param timeStamp time when message was send
 		 */
-		void receivedMultiIM(const QString &chat, const QString &body, const QString &messageId, const QString &user);
+		void receivedMultiIM(const QString &chat, const QString &body, const QString &messageId, const QString &user, const QDateTime &timeStamp);
 		/**
 		 * This is emitted when an Id of the last outgoing message is known
 		 * @param id The ID of that message
@@ -529,6 +569,17 @@ class Skype : public QObject
 		 * Note: the group should be closed when all it's calls are closed
 		 */
 		void groupCall(const QString &callId, const QString &groupId);
+		/**
+		 * Emitted when we recive auth request
+		 * @param contactId who send request
+		 * @parma info info in request
+		 */
+		void receivedAuth(const QString &contactId, const QString &info);
+
+		///Video section
+
+		void startReceivingVideo(const QString &callId);
+		void stopReceivingVideo(const QString &callId);
 };
 
 #endif
