@@ -25,7 +25,6 @@
 #include "skypecalldialog.h"
 #include "skypechatsession.h"
 #include "skypeconference.h"
-#include "skypeactionhandleradaptor.h"
 
 #include <qstring.h>
 #include <kopetemetacontact.h>
@@ -37,9 +36,12 @@
 #include <kmessagebox.h>
 #include <klocale.h>
 #include <kconfigbase.h>
-#include <Q3Dict>
+#include <QHash>
 #include <kopetemessage.h>
 #include <qprocess.h>
+#include <kopeteaddedinfoevent.h>
+
+#include <QDateTime>
 
 class SkypeAccountPrivate {
 	public:
@@ -76,13 +78,13 @@ class SkypeAccountPrivate {
 		///Do we wait before connecting?
 		int waitBeforeConnect;
 		///List of chat all chat sessions
-		Q3Dict<SkypeChatSession> sessions;
+		QHash<QString, SkypeChatSession*> sessions;
 		///Last used chat session
 		SkypeChatSession *lastSession;
 		///List of the conference calls
-		Q3Dict<SkypeConference> conferences;
+		QHash<QString, SkypeConference*> conferences;
 		///List of existing calls
-		Q3Dict<SkypeCallDialog> calls;
+		QHash<QString, SkypeCallDialog*> calls;
 		///Shall chat window leave the chat whenit is closed
 		bool leaveOnExit;
 		///Executed before making the call
@@ -107,8 +109,6 @@ SkypeAccount::SkypeAccount(SkypeProtocol *protocol, const QString& accountID) :
 
 	//the d pointer
 	d = new SkypeAccountPrivate(*this);
-	d->calls.setAutoDelete(false);
-	d->conferences.setAutoDelete(false);
 	//remember the protocol, it will be needed
 	d->protocol = protocol;
 
@@ -120,7 +120,7 @@ SkypeAccount::SkypeAccount(SkypeProtocol *protocol, const QString& accountID) :
 	setCallControl(config->readEntry("CallControl", false));
 	setPings(config->readEntry("Pings", true));
 	setBus(config->readEntry("Bus", 1));
-	setLaunchTimeout(config->readEntry("LaunchTimeout", 30));
+	setLaunchTimeout(config->readEntry("LaunchTimeout", 70));
 	d->myName = config->readEntry("MyselfName", "Skype");
 	setSkypeCommand(config->readEntry("SkypeCommand", "skype"));
 	setWaitBeforeConnect(config->readEntry("WaitBeforeConnect", 0));
@@ -148,27 +148,23 @@ SkypeAccount::SkypeAccount(SkypeProtocol *protocol, const QString& accountID) :
 	QObject::connect(&d->skype, SIGNAL(statusConnecting()), this, SLOT(statusConnecting()));
 	QObject::connect(&d->skype, SIGNAL(newUser(const QString&, int)), this, SLOT(newUser(const QString&, int)));
 	QObject::connect(&d->skype, SIGNAL(contactInfo(const QString&, const QString& )), this, SLOT(updateContactInfo(const QString&, const QString& )));
-	QObject::connect(&d->skype, SIGNAL(receivedIM(const QString&, const QString&, const QString& )), this, SLOT(receivedIm(const QString&, const QString&, const QString& )));
+	QObject::connect(&d->skype, SIGNAL(receivedIM(const QString&, const QString&, const QString&, const QDateTime&)), this, SLOT(receivedIm(const QString&, const QString&, const QString&, const QDateTime&)));
 	QObject::connect(&d->skype, SIGNAL(gotMessageId(const QString& )), this, SLOT(gotMessageId(const QString& )));//every time some ID is known inform the contacts
 	QObject::connect(&d->skype, SIGNAL(newCall(const QString&, const QString&)), this, SLOT(newCall(const QString&, const QString&)));
 	QObject::connect(&d->skype, SIGNAL(setMyselfName(const QString&)), this, SLOT(setMyselfName(const QString& )));
-	QObject::connect(&d->skype, SIGNAL(receivedMultiIM(const QString&, const QString&, const QString&, const QString& )), this, SLOT(receiveMultiIm(const QString&, const QString&, const QString&, const QString& )));
+	QObject::connect(&d->skype, SIGNAL(receivedMultiIM(const QString&, const QString&, const QString&, const QString&, const QDateTime&)), this, SLOT(receiveMultiIm(const QString&, const QString&, const QString&, const QString&, const QDateTime&)));
 	QObject::connect(&d->skype, SIGNAL(outgoingMessage(const QString&, const QString&)), this, SLOT(sentMessage(const QString&, const QString& )));
 	QObject::connect(&d->skype, SIGNAL(groupCall(const QString&, const QString& )), this, SLOT(groupCall(const QString&, const QString& )));
+	QObject::connect(&d->skype, SIGNAL(receivedAuth(const QString &, const QString &)), this, SLOT(receivedAuth(const QString &, const QString &)));
 	QObject::connect(Kopete::ContactList::self(), SIGNAL(groupRemoved (Kopete::Group *)), this, SLOT(deleteGroup (Kopete::Group *) ) );
 	QObject::connect(Kopete::ContactList::self(), SIGNAL(groupRenamed (Kopete::Group *, const QString& )), this, SLOT(renameGroup (Kopete::Group *, const QString& )) );
 
-	//Set SkypeActionHandler for web SkypeButtons
-	new SkypeActionHandlerAdaptor(this);
-	QDBusConnection::sessionBus().registerObject("/SkypeActionHandler", this);
-
 	//set values for the connection (should be updated if changed)
 	d->skype.setValues(launchType, author);
 	setHitchHike(config->readEntry("Hitch", true));
 	setMarkRead(config->readEntry("MarkRead", true));//read the modes of account
 	d->callWindowTimeout = config->readEntry("CloseWindowTimeout", 4);
 	setPings(config->readEntry("Pings", true));
-	d->sessions.setAutoDelete(false);
 	d->lastSession = 0L;
 	d->callCount = 0;
 }
@@ -210,33 +206,42 @@ void SkypeAccount::setAway(bool away, const QString &reason) {
 
 void SkypeAccount::setOnlineStatus(const Kopete::OnlineStatus &status, const Kopete::StatusMessage &reason, const OnlineStatusOptions& options) {
 	kDebug(SKYPE_DEBUG_GLOBAL);
-
-	if (status == d->protocol->Online)
-		d->skype.setOnline();//Go online
-	else if (status == d->protocol->Offline)
-		d->skype.setOffline();//Go offline
-	else if (status == d->protocol->Away)
-		d->skype.setAway();
-	else if (status == d->protocol->NotAvailable)
-		d->skype.setNotAvailable();
-	else if (status == d->protocol->DoNotDisturb)
-		d->skype.setDND();
-	else if (status == d->protocol->Invisible)
-		d->skype.setInvisible();
-	else if (status == d->protocol->SkypeMe)
-		d->skype.setSkypeMe();
-	else
-		kDebug(SKYPE_DEBUG_GLOBAL) << "Unknown online status";//Just a warning that I do not know that status
-
-	///TODO: Port to kde4
-	Q_UNUSED(reason);
 	Q_UNUSED(options);
+	if (status == d->protocol->Online){
+		d->skype.setOnline();//Go online
+		setStatusMessage(reason);
+	}
+	else if (status == d->protocol->Offline){
+		if (!reason.isEmpty()) setStatusMessage(reason);
+		d->skype.setOffline();//Go offline
+	}
+	else if (status == d->protocol->Away){
+		d->skype.setAway();
+		setStatusMessage(reason);
+	}
+	else if (status == d->protocol->NotAvailable){
+		d->skype.setNotAvailable();
+		setStatusMessage(reason);
+	}
+	else if (status == d->protocol->DoNotDisturb){
+		d->skype.setDND();
+		setStatusMessage(reason);
+	}
+	else if (status == d->protocol->Invisible){
+		d->skype.setInvisible();
+		setStatusMessage(reason);
+	}
+	else if (status == d->protocol->SkypeMe){
+		d->skype.setSkypeMe();
+		setStatusMessage(reason);
+	}
+	else
+		kDebug(SKYPE_DEBUG_GLOBAL) << "Unknown online status";//Just a warning that I do not know that status;
 }
 
 void SkypeAccount::setStatusMessage(const Kopete::StatusMessage &statusMessage)
 {
-	///TODO: Port to kde4
-	Q_UNUSED(statusMessage);
+	d->skype.setUserProfileRichMoodText(statusMessage.message());
 }
 
 void SkypeAccount::disconnect() {
@@ -472,9 +477,9 @@ bool SkypeAccount::userHasChat(const QString &userId) {
 		return false;//if it does not exist it can not have a chat opened
 }
 
-void SkypeAccount::receivedIm(const QString &user, const QString &message, const QString &messageId) {
+void SkypeAccount::receivedIm(const QString &user, const QString &message, const QString &messageId, const QDateTime &timeStamp) {
 	kDebug(SKYPE_DEBUG_GLOBAL) << "User: " << user << ", message: " << message;
-	getContact(user)->receiveIm(message, getMessageChat(messageId));//let the contact show the message
+	getContact(user)->receiveIm(message, getMessageChat(messageId), timeStamp);//let the contact show the message
 }
 
 void SkypeAccount::setScanForUnread(bool value) {
@@ -521,6 +526,8 @@ void SkypeAccount::newCall(const QString &callId, const QString &userId) {
 		QObject::connect(&d->skype, SIGNAL(skypeOutInfo(int, const QString& )), dialog, SLOT(skypeOutInfo(int, const QString& )));
 		QObject::connect(dialog, SIGNAL(updateSkypeOut()), &d->skype, SLOT(getSkypeOut()));
 		QObject::connect(dialog, SIGNAL(callFinished(const QString& )), this, SLOT(removeCall(const QString& )));
+		QObject::connect(&d->skype, SIGNAL(startReceivingVideo(const QString &)), dialog, SLOT(startReceivingVideo(const QString &)));
+		QObject::connect(&d->skype, SIGNAL(stopReceivingVideo(const QString &)), dialog, SLOT(stopReceivingVideo(const QString &)));
 
 		dialog->show();//Show Call dialog
 
@@ -653,11 +660,11 @@ void SkypeAccount::setChatId(const QString &oldId, const QString &newId, SkypeCh
 }
 
 bool SkypeAccount::chatExists(const QString &chat) {
-	return d->sessions.find(chat);
+	return d->sessions.value(chat);
 }
 
-void SkypeAccount::receiveMultiIm(const QString &chatId, const QString &body, const QString &messageId, const QString &user) {
-	SkypeChatSession *session = d->sessions.find(chatId);
+void SkypeAccount::receiveMultiIm(const QString &chatId, const QString &body, const QString &messageId, const QString &user, const QDateTime &timeStamp) {
+	SkypeChatSession *session = d->sessions.value(chatId);
 
 	if (!session) {
 		QStringList users = d->skype.getChatUsers(chatId);
@@ -672,6 +679,7 @@ void SkypeAccount::receiveMultiIm(const QString &chatId, const QString &body, co
 	Kopete::Message mes(getContact(user), myself());
 	mes.setDirection(Kopete::Message::Inbound);
 	mes.setPlainBody(body);
+	mes.setTimestamp(timeStamp);
 	session->appendMessage(mes);
 
 	Q_UNUSED(messageId);
@@ -696,7 +704,7 @@ void SkypeAccount::gotMessageId(const QString &messageId) {
 void SkypeAccount::sentMessage(const QString &body, const QString &chat) {
 	kDebug(SKYPE_DEBUG_GLOBAL) << "chat: " << chat;
 
-	SkypeChatSession *session = d->sessions.find(chat);
+	SkypeChatSession *session = d->sessions.value(chat);
 	const QStringList &users = d->skype.getChatUsers(chat);
 	QList<Kopete::Contact*> *recv = 0L;
 
@@ -742,14 +750,14 @@ void SkypeAccount::groupCall(const QString &callId, const QString &groupId) {
 		return;
 
 	SkypeConference *conf;
-	if (!(conf = d->conferences[groupId])) {//does it already exist?
+	if (!(conf = d->conferences.value(groupId))) {//does it already exist?
 		conf = new SkypeConference(groupId);//no, create one then..
 		d->conferences.insert(groupId, conf);
 
 		QObject::connect(conf, SIGNAL(removeConference(const QString& )), this, SLOT(removeCallGroup(const QString& )));
 	}
 
-	conf->embedCall(d->calls[callId]);
+	conf->embedCall(d->calls.value(callId));
 }
 
 void SkypeAccount::removeCall(const QString &callId) {
@@ -1010,11 +1018,80 @@ void SkypeAccount::setDisplayName(const QString &user, const QString &name) {
 	d->skype.setDisplayName(user, name);
 }
 
+void SkypeAccount::receivedAuth(const QString &user, const QString &info) {
+	Kopete::AddedInfoEvent* event = new Kopete::AddedInfoEvent(user, this);
+
+	QObject::connect( event, SIGNAL(actionActivated(uint)), this, SLOT(authEvent(uint)) );
+
+	Kopete::AddedInfoEvent::ShowActionOptions actions = Kopete::AddedInfoEvent::AuthorizeAction;
+	actions |= Kopete::AddedInfoEvent::BlockAction;
+	actions |= Kopete::AddedInfoEvent::InfoAction;
+
+	Kopete::Contact * ct = contacts().value( user );
+	if( !ct || !ct->metaContact() || ct->metaContact()->isTemporary() )
+		actions |= Kopete::AddedInfoEvent::AddAction;
+
+	if( ct )
+		event->setContactNickname( ct->nickName() );
+
+	event->showActions( actions );
+
+	event->setAdditionalText(info);
+	event->sendEvent();
+}
+
+void SkypeAccount::authEvent(uint actionId) {
+	Kopete::AddedInfoEvent *event = dynamic_cast<Kopete::AddedInfoEvent *>(sender());
+
+	if ( !event )
+		return;
+
+	switch ( actionId ) {
+		case Kopete::AddedInfoEvent::AddContactAction:
+			event->addContact();
+			break;
+		case Kopete::AddedInfoEvent::AuthorizeAction:
+			authorizeUser(event->contactId());
+			break;
+		case Kopete::AddedInfoEvent::BlockAction:
+			blockUser(event->contactId());
+			break;
+		case Kopete::AddedInfoEvent::InfoAction:
+			userInfo(event->contactId());
+			break;
+	}
+}
+
+void SkypeAccount::startSendingVideo(const QString &callId) {
+	d->skype.startSendingVideo(callId);
+}
+
+void SkypeAccount::stopSendingVideo(const QString &callId) {
+	d->skype.stopSendingVideo(callId);
+}
+
+void SkypeAccount::userInfo(const QString &user) {
+	kDebug(SKYPE_DEBUG_GLOBAL) << user;
+	if ( ! contact(user) ) {
+		addContact(user, d->skype.getDisplayName(user), 0L, Temporary);//create a temporary contact
+		if ( ! contact(user) ) {
+			KMessageBox::error(0L, i18n("Cannot open info about user %1", user), i18n("Skype protocol"));
+			return;//contact arent in contact list - skip it
+		}
+	}
+	contact(user)->slotUserInfo();//Open user info dialog
+}
+
 void SkypeAccount::SkypeActionHandler(const QString &message) {
 	kDebug(SKYPE_DEBUG_GLOBAL) << message;
 
-	if ( message.isEmpty() )
+	//TODO:
+	//Add all actions from https://developer.skype.com/Docs/ApiDoc/Skype_URI_handler
+
+	if ( message.isEmpty() ) {
+		KMessageBox::error(0L, i18n("Unknown action from SkypeActionHandler"), i18n("Skype protocol"));
 		return;//Empty message
+	}
 
 	QString command;
 	QString user;
@@ -1030,32 +1107,37 @@ void SkypeAccount::SkypeActionHandler(const QString &message) {
 		user = message.section(":", -1).section("?", 0, 0).trimmed();
 		if ( command.isEmpty() ) //set default double click action = open chat window
 			command = "chat";
-	} else
-		return;//Unknow message
+	} else {
+		KMessageBox::error(0L, i18n("Unknown action from SkypeActionHandler"), i18n("Skype protocol"));
+		return;//Unknown message
+	}
 
-	if ( command.isEmpty() || user.isEmpty() )
-		return;//Unknow message
+	if ( command.isEmpty() || user.isEmpty() ) {
+		KMessageBox::error(0L, i18n("Unknown action from SkypeActionHandler"), i18n("Skype protocol"));
+		return;//Unknown message
+	}
 
 	kDebug(SKYPE_DEBUG_GLOBAL) << "user:" << user << "command:" << command;
 
 	if ( command == "add" ) {
 		//TODO: Open add user dialog
+		KMessageBox::error(0L, i18n("Add contact from SkypeActionHandler is not supported yet"), i18n("Skype protocol"));
 	} else if ( command == "call" ) {
+		//TODO: Add support for conference call
 		makeCall(user);//Start call with user
 	} else if ( command == "chat" ) {
+		//TODO: Add support for multichat
 		chatUser(user);//Open chat window
 	} else if ( command == "sendfile" ) {
 		openFileTransfer(user);//Open file transfer dialog
-	} else if ( command == "userinfo" ) {//TODO: Open option dialog (with all thisa options instead userinfo) and support unknow contacts who arent in contact list
-		if ( ! contact(user) ) {
-			addContact(user, d->skype.getDisplayName(user), 0L, Temporary);//create a temporary contact
-			if ( ! contact(user) )
-				return;//contact arent in contact list - skip it
-		}
-		contact(user)->slotUserInfo();//Open user info dialog
-		//TODO: dont use slot, it freeze dbus, better create new signal
+	} else if ( command == "voicemail" ) {
+		//TODO: Send voicemail
+		KMessageBox::error(0L, i18n("Send voicemail from SkypeActionHandler is not supported yet"), i18n("Skype protocol"));
+	} else if ( command == "userinfo" ) {//TODO: Open option dialog (with all thisa options instead userinfo) and support unknown contacts who arent in contact list
+		userInfo(user);
 	} else {
-		kDebug(SKYPE_DEBUG_GLOBAL) << "Unknow command";
+		kDebug(SKYPE_DEBUG_GLOBAL) << "Unknown command";
+		KMessageBox::error(0L, i18n("Unknown action from SkypeActionHandler"), i18n("Skype protocol"));
 		return;//Unknow command
 	}
 }
