@@ -46,6 +46,7 @@
 #include "conferencetask.h"
 #include "sendauthresptask.h"
 #include "pingtask.h"
+#include "alivetask.h"
 #include "yabtask.h"
 #include "modifyyabtask.h"
 #include "chatsessiontask.h"
@@ -59,6 +60,7 @@
 
 using namespace KNetwork;
 
+
 class Client::ClientPrivate
 {
 public:
@@ -74,7 +76,8 @@ public:
 	int error;
 	QString errorString;
 	QString errorInformation;
-	
+	QStringList stealthedBuddies;
+	QStringList unstealthedBuddies;
 	// tasks
 	bool tasksInitialized;
 	LoginTask * loginTask;
@@ -88,6 +91,7 @@ public:
 	YABTask *yabTask;
 	FileTransferNotifierTask *fileTransferTask;
 	YahooChatTask *yahooChatTask;
+	ReceiveFileTask *receiveFileTask;
 
 	// Connection data
 	uint sessionID;
@@ -124,9 +128,12 @@ Client::Client(QObject *par) :QObject(par)
 
 	m_pingTimer = new QTimer( this );
 	QObject::connect( m_pingTimer, SIGNAL( timeout() ), this, SLOT( sendPing() ) );
-
+	m_aliveTimer = new QTimer( this );
+	QObject::connect( m_aliveTimer, SIGNAL( timeout() ), this, SLOT( sendAlive() ) );
+	
 	QObject::connect( d->loginTask, SIGNAL( haveSessionID( uint ) ), SLOT( lt_gotSessionID( uint ) ) );
 	QObject::connect( d->loginTask, SIGNAL( buddyListReady() ), SLOT( processPictureQueue() ) );
+	QObject::connect( d->loginTask, SIGNAL( buddyListReady() ), SLOT( processStealthQueue() ) );
 	QObject::connect( d->loginTask, SIGNAL( loginResponse( int, const QString& ) ), 
 				SLOT( slotLoginResponse( int, const QString& ) ) );
 	QObject::connect( d->loginTask, SIGNAL( haveCookies() ), SLOT( slotGotCookies() ) );
@@ -159,7 +166,7 @@ void Client::connect( const QString &host, const uint port, const QString &userI
 	QObject::connect( d->stream, SIGNAL( connected() ), this, SLOT( cs_connected() ) );
 	QObject::connect( d->stream, SIGNAL( error(int) ), this, SLOT( streamError(int) ) );
 	QObject::connect( d->stream, SIGNAL( readyRead() ), this, SLOT( streamReadyRead() ) );
-	QObject::connect( d->stream, SIGNAL( disconnected() ), this, SLOT( streamDisconnected() ) );
+	QObject::connect( d->stream, SIGNAL( connectionClosed() ), this, SLOT( streamDisconnected() ) );
 	
 	d->stream->connectToServer( host, false );
 }
@@ -177,6 +184,7 @@ void Client::cs_connected()
 
 	d->loginTask->setStateOnConnect( (d->statusOnConnect == Yahoo::StatusInvisible) ? Yahoo::StatusInvisible : Yahoo::StatusAvailable );
 	d->loginTask->go();
+	processStealthQueue();
 	d->active = true;
 }
 
@@ -184,6 +192,7 @@ void Client::close()
 {
 	kDebug(YAHOO_RAW_DEBUG) ;
 	m_pingTimer->stop();
+	m_aliveTimer->stop();
 	if( d->active )
 	{
 		LogoffTask *lt = new LogoffTask( d->root );
@@ -275,8 +284,15 @@ void Client::slotLoginResponse( int response, const QString &msg )
 			changeStatus( d->statusOnConnect, d->statusMessageOnConnect, Yahoo::StatusTypeAway );
 		d->statusMessageOnConnect.clear();
 		setStatus( d->statusOnConnect );
-		/* only send a ping every hour. we get disconnected otherwise */
-		m_pingTimer->start( 60 * 60 * 1000 );
+		/* YM Client sends a alert every minute
+		* If we time out and try to log out and log in
+		* we can lose messages therefore we should set this
+		* to the same as the Yahoo Messenger client
+		*. so as we dont get disconnected
+		* Also it sends a PING at every hour.
+		*/
+		m_aliveTimer->start( 1 * 60 * 1000 );
+		m_pingTimer->start( 59 * 60 * 1000 );
 		initTasks();
 	} else {
 		d->active = false;
@@ -442,6 +458,17 @@ void Client::sendPing()
 	pt->go( true );
 }
 
+void Client::sendAlive()
+{
+	if( !d->active )
+	{
+		kDebug(YAHOO_RAW_DEBUG) << "Disconnected. NOT sending a ALIVE.";
+		return;
+	}
+	kDebug(YAHOO_RAW_DEBUG) << "Sending a ALIVE.";
+	AliveTask *at = new AliveTask( d->root );
+	at->go( true );
+}
 // ***** Contactlist handling *****
 
 void Client::stealthContact(QString const &userId, Yahoo::StealthMode mode, Yahoo::StealthStatus state)
@@ -547,7 +574,7 @@ void Client::uploadPicture( KUrl url )
 	spt->setType( SendPictureTask::UploadPicture );
 	spt->setFilename( url.fileName() );
 	if ( url.isLocalFile() )
-		spt->setPath( url.path() );
+		spt->setPath( url.toLocalFile() );
 	else
 		spt->setPath( url.url() );
 	spt->go( true );
@@ -588,6 +615,29 @@ void Client::setPictureStatus( Yahoo::PictureStatus status )
 	spt->go( true );
 }
 
+// **** Stealth Handling **** michaelacole
+void Client::processStealthQueue()
+{
+	if( d->buddyListReady ){
+		while (!d->unstealthedBuddies.isEmpty())
+			{
+			QString it;
+			it = d->unstealthedBuddies.takeFirst();
+			kDebug(YAHOO_RAW_DEBUG) << "unstealthed setting set on" << it;
+			stealthContact( it, Yahoo::StealthOffline , Yahoo::StealthNotActive );
+			}
+
+		while (!d->stealthedBuddies.isEmpty())
+			{
+			QString it;
+			it = d->stealthedBuddies.takeFirst();
+			kDebug(YAHOO_RAW_DEBUG) << "stealthed setting set on" << it ;
+			stealthContact( it, Yahoo::StealthPermOffline , Yahoo::StealthActive );
+			}
+	}
+}
+
+
 // ***** Webcam handling *****
 
 void Client::requestWebcam( const QString &userId )
@@ -717,6 +767,16 @@ void Client::notifyError( const QString &info, const QString & errorString, LogL
 	emit error( level );
 }
 
+void Client::notifyStealthedBuddies( const QStringList &buddies)
+{
+	d->stealthedBuddies = buddies;
+}
+
+void Client::notifyUnstealthedBuddies( const QStringList &buddies)
+{
+	d->unstealthedBuddies = buddies;
+}
+
 QString Client::userId()
 {
 	return d->user;
@@ -960,6 +1020,8 @@ void Client::deleteTasks()
 	d->fileTransferTask = 0;
 	d->yahooChatTask->deleteLater();
 	d->yahooChatTask = 0;
+	d->receiveFileTask->deleteLater();
+	d->receiveFileTask = 0;
 }
 
 #include "client.moc"
